@@ -238,3 +238,87 @@ def plot_decay_solution(solutions, model_generator, sigma, prefix,
         if not test:
             plt.savefig(filename, bbox_inches='tight')
         # plt.show()
+
+
+def plot_experiment_equipment(tolerances, res, prefix, fsize=32, linewidth=5,
+                              title="Variance of MUD Error", test=False):
+    print("Plotting equipment differences...")
+    plt.figure(figsize=(10, 10))
+    for _res in res:
+        _prefix, _in, _rm, _re = _res
+        regression_err_mean, slope_err_mean, regression_err_vars, slope_err_vars, sd_means, sd_vars, num_sensors = _re
+        plt.plot(tolerances, regression_err_mean, label=f"{_prefix:10s} slope: {slope_err_mean:1.4f}", lw=linewidth)
+        plt.scatter(tolerances, sd_means, marker='x', lw=20)
+
+    plt.yscale('log')
+    plt.xscale('log')
+    plt.Axes.set_aspect(plt.gca(), 1)
+    plt.ylim(2E-3, 2E-2)
+    # plt.ylabel("Absolute Error", fontsize=fsize)
+    plt.xlabel('Tolerance', fontsize=fsize)
+    plt.legend()
+    plt.title(f"Mean of MUD Error for N={num_sensors}", fontsize=1.25 * fsize)
+    if not test:
+        plt.savefig(f'{prefix}_convergence_mud_std_mean.png', bbox_inches='tight')
+    # plt.show()
+
+    plt.figure(figsize=(10, 10))
+    for _res in res:
+        _prefix, _in, _rm, _re = _res
+        regression_err_mean, slope_err_mean, regression_err_vars, slope_err_vars, sd_means, sd_vars, num_sensors = _re
+        plt.plot(tolerances, regression_err_vars, label=f"{_prefix:10s} slope: {slope_err_vars:1.4f}", lw=linewidth)
+        plt.scatter(tolerances, sd_vars, marker='x', lw=20)
+    plt.xscale('log')
+    plt.yscale('log')
+    plt.ylim(2E-5, 2E-4)
+    plt.Axes.set_aspect(plt.gca(), 1)
+    # plt.ylabel("Absolute Error", fontsize=fsize)
+    plt.xlabel('Tolerance', fontsize=fsize)
+    plt.legend()
+    plt.title(title, fontsize=1.25 * fsize)
+    if not test:
+        plt.savefig(f'{prefix}_convergence_mud_std_var.png', bbox_inches='tight')
+    # plt.show()
+
+
+def plot_experiment_measurements(measurements, res, prefix, fsize=32, linewidth=5, xlabel='Number of Measurements', test=False, legend=False):
+    print("Plotting measurement experiments.")
+    plt.figure(figsize=(10, 10))
+    for _res in res:
+        _prefix, _in, _rm, _re = _res
+        regression_mean, slope_mean, regression_vars, slope_vars, means, variances = _rm
+        plt.plot(measurements, regression_mean, label=f"{_prefix:10s} slope: {slope_mean:1.4f}", lw=linewidth)
+        plt.scatter(measurements, means, marker='x', lw=20)
+    plt.xscale('log')
+    plt.yscale('log')
+    plt.Axes.set_aspect(plt.gca(), 1)
+    plt.ylim(0.9 * min(means), 1.3 * max(means))
+    plt.ylim(2E-3, 2E-1)
+    plt.xlabel(xlabel, fontsize=fsize)
+    if legend:
+        plt.legend(fontsize=fsize * 0.8)
+    # plt.ylabel('Absolute Error in MUD', fontsize=fsize)
+    plt.title("$\\mathrm{\\mathbb{E}}(|\\lambda^\\mathrm{MUD} - \\lambda^\\dagger|)$", fontsize=1.25 * fsize)
+    if not test:
+        plt.savefig(f'{prefix}_convergence_mud_obs_mean.png', bbox_inches='tight')
+    # plt.show()
+
+    plt.figure(figsize=(10, 10))
+    for _res in res:
+        _prefix, _in, _rm, _re = _res
+        regression_mean, slope_mean, regression_vars, slope_vars, means, variances = _rm
+        plt.plot(measurements, regression_vars, label=f"{_prefix:10s} slope: {slope_vars:1.4f}", lw=linewidth)
+        plt.scatter(measurements, variances, marker='x', lw=20)
+    plt.xscale('log')
+    plt.yscale('log')
+    plt.Axes.set_aspect(plt.gca(), 1)
+    plt.ylim(0.9 * min(variances), 1.3 * max(variances))
+    plt.ylim(1E-5, 1E-3)
+    plt.xlabel(xlabel, fontsize=fsize)
+    if legend:
+        plt.legend(fontsize=fsize * 0.8)
+    # plt.ylabel('Absolute Error in MUD', fontsize=fsize)
+    plt.title("$\\mathrm{Var}(|\\lambda^\\mathrm{MUD} - \\lambda^\\dagger|)$", fontsize=1.25 * fsize)
+    if not test:
+        plt.savefig(f'{prefix}_convergence_mud_obs_var.png', bbox_inches='tight')
+    # plt.show()
